@@ -61,7 +61,7 @@ static void controlpad_preset_edit_ok(GtkWidget *widget, gpointer *data) {
   GtkWidget *entry_number = gtk_object_get_data(GTK_OBJECT(widget),
                                                         "entry_number");
   GtkWidget *button = gtk_object_get_data(GTK_OBJECT(widget), "button");
-  int n = (int)gtk_object_get_data(GTK_OBJECT(widget), "number");
+  int n = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget), "number"));
 
   free(session->preset_names[n]);
   session->preset_names[n] = strdup(gtk_entry_get_text(GTK_ENTRY(entry_name)));
@@ -146,7 +146,7 @@ static gint controlpad_preset_edit_cb(GtkWidget *widget, GdkEventButton *event,
     gtk_object_set_data(GTK_OBJECT(button), "entry_name", entry_name);
     gtk_object_set_data(GTK_OBJECT(button), "entry_number", entry_number);
     gtk_object_set_data(GTK_OBJECT(button), "button", widget);
-    gtk_object_set_data(GTK_OBJECT(button), "number", (gpointer) n);
+    gtk_object_set_data(GTK_OBJECT(button), "number", GINT_TO_POINTER(n));
     gtk_object_set_data(GTK_OBJECT(button), "window", window);
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		       GTK_SIGNAL_FUNC(controlpad_preset_edit_ok),
