@@ -35,20 +35,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include <limits.h>
 
 /* own header files */
 #include "globals.h"
 #include "session.h"
 #include "settings.h"
-#include "optionsparser.h"
 #include "calleridparser.h"
 #include "util.h"
 #include "callerid.h"
 
-extern FILE *options_in;
-int options_parse(void *session);
+#include "isdn.h"
+#include "isdnlexer.h"
+#include "isdnparser.h"
+#include "isdntree.h"
+
 extern FILE *callerid_in;
 int callerid_parse(void *session);
+extern FILE *isdn_in;
+int isdn_parse();
 
 /*
  * sets an option for the specified session
@@ -56,7 +61,7 @@ int callerid_parse(void *session);
  * used as callback from yyparse()
  */
 void settings_option_set(session_t *session, char *option, char *value) {
-  int i_value = -1; /* set value to 0 or 1 if appropriate */
+  int i_value = INT_MIN; /* set value to 0 or 1 if appropriate */
 
   if (!strcmp(value, "0") || !strcasecmp(value, "false") ||
       !strcasecmp(value, "off")) {
@@ -69,7 +74,7 @@ void settings_option_set(session_t *session, char *option, char *value) {
     i_value = strtol(value, NULL, 0);
 
   /* Settings */
-  if (i_value != -1) {
+  if (i_value != INT_MIN) {
     if (!strcmp(option, "HistorySize")) {
       session->dial_number_history_maxlen = i_value;
     }
@@ -185,34 +190,104 @@ void settings_option_set(session_t *session, char *option, char *value) {
 }
 
 /*
- * Read options from options file (in dotfile directory)
+ * Read options from options file and isdn4linux config
  *
- * Here, we just parse the config file and _change_ options. the defaults
+ * Here, we parse the config file and _change_ options. The defaults
  * will be all set at init time
  */
 void settings_options_read(session_t *session) {
   char *homedir;
   char *filename;
 
+  /* read isdn4linux config */
+  isdn_lexer_init(ISDN_CONFIG_FILENAME);
+  if (!isdn_in) {
+    fprintf(stderr, "Warning: Couldn't read ISDN config file.\n");
+  } else {
+    isdn_tree_node_t* node;
+    
+    isdn_parse();
+    isdn_lexer_deinit();
+    if (debug > 1) {
+      printf("ISDN config file (%s) contents:\n", ISDN_CONFIG_FILENAME);
+      isdn_tree_dump();
+    }
+    /* get calls file name if possible */
+    node = isdn_tree;
+    while (node != NULL) {
+      if (node->type == ISDN_NODE_TYPE_SECTION &&
+	  !strcmp(node->name, "ISDNLOG"))
+      {
+	isdn_tree_node_t* entry = node->content.section;
+
+	while (entry != NULL ) {
+	  if (entry->type == ISDN_NODE_TYPE_ENTRY &&
+	      !strcmp(entry->name, "LOGFILE"))
+	  {
+	    if (isdn_calls_filename_from_config)
+	      free(isdn_calls_filename_from_config);
+	    isdn_calls_filename_from_config = strdup(entry->content.value);
+	  }
+	  entry = entry->next;
+	}
+      }
+      node = node->next;
+    }
+    
+    isdn_tree_free();
+  }
+
   if (!(homedir = get_homedir())) {
     fprintf(stderr, "Warning: Couldn't get home dir.\n");
     return;
   }
 
+  /* dotfile */
   if (asprintf(&filename, "%s/." PACKAGE "/%s",
-	       homedir, SETTINGS_OPTIONS_FILENAME) < 0) {
+	       homedir, SETTINGS_OPTIONS_FILENAME) < 0)
+  {
     fprintf(stderr,
 	    "Warning: Couldn't allocate memory for options filename.\n");
     return;
   }
 
-  if ((options_in = fopen(filename, "r"))) {
-    options_parse(session);
-    if (fclose(options_in) == EOF) {
-      fprintf(stderr, "Warning: Couldn't close options file.\n");
-    }
-  } else if (debug) {
+  isdn_lexer_init(filename);
+  if (!isdn_in) {
     fprintf(stderr, "Warning: No options file available.\n");
+  } else {
+    isdn_tree_node_t* node;
+
+    isdn_parse();
+    isdn_lexer_deinit();
+    node = isdn_tree;
+    if (debug > 1) {
+      printf("Options file (%s) contents:\n", filename);
+      isdn_tree_dump();
+    }
+    /* actually set options */
+    while (node != NULL) {
+      switch (node->type) {
+        case ISDN_NODE_TYPE_ENTRY:
+	  if (debug)
+	    printf("Setting \"%s\" to \"%s\"...\n",
+		   node->name, node->content.value);
+	  settings_option_set(session, node->name, node->content.value);
+	  break;
+        case ISDN_NODE_TYPE_SECTION:
+	  fprintf(stderr, "Warning: Unexpected section \"%s\".\n", node->name);
+	  break;
+        case ISDN_NODE_TYPE_SUBSECTION:
+	  fprintf(stderr,
+	          "Warning: Unexpected subsection \"%s\".\n", node->name);
+          break;
+        default:
+	  fprintf(stderr, "Unknown ISDN_NODE_TYPE\n");
+      }
+    
+      node = node->next;
+    }
+
+    isdn_tree_free();
   }
   free(filename);
 }
